package com.wikihouse.wildcats0201.jdbcchart.reflectionutil;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.ClassNotFoundExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.IllegalAccessExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.InstantiationExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.NoSuchMethodExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.SecurityExceptionWrapper;

/**
 * <B>ClassWrapper </B>. ClassbsONXłB
 * 
 * @see java.lang.Class
 * @author $Author: wildcats $
 * @version $Revision: 5 $
 */
public class ClassWrapper {

    private Class clazz;

    private ClassWrapper() {
    }

    public ClassWrapper(String className) {
        this(getClass(className));
    }

    private static Class getClass(String className) {
        Class clazz = null;
        try {
            clazz = Class.forName(className);
        } catch (java.lang.ClassNotFoundException e) {
            throw new ClassNotFoundExceptionWrapper(e);
        }
        return clazz;
    }

    public ClassWrapper(Class clazz) {
        this.clazz = clazz;
    }

    public Class get() {
        return this.clazz;
    }

    public Object newInstance() {
        Object obj = null;
        try {
            obj = clazz.newInstance();
        } catch (java.lang.InstantiationException e) {
            throw new InstantiationExceptionWrapper(e);
        } catch (java.lang.IllegalAccessException e) {
            throw new IllegalAccessExceptionWrapper(e);
        }
        return obj;
    }

    private ConstructorWrapper getConstructorWrapper(Constructor constructor) {
        return new ConstructorWrapper(constructor);
    }

    public ConstructorWrapper getConstructor(Class clazz) {
        return getConstructor(new Class[] { clazz });
    }

    public ConstructorWrapper getConstructor(Class[] classes) {
        Constructor constructor = null;
        try {
            constructor = this.clazz.getConstructor(classes);
        } catch (java.lang.SecurityException e) {
            throw getSecurityException(e);
        } catch (java.lang.NoSuchMethodException e) {
            throw getNoSucheMethodException(e);
        }
        return getConstructorWrapper(constructor);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return clazz.getName();
    }

    public Method[] getMethods() {
        return this.clazz.getMethods();
    }

    public Method[] getMethods(String methodName) {
        List list = new LinkedList();
        for (Iterator it = Arrays.asList(this.getMethods()).iterator(); it
                .hasNext();) {
            Method method = (Method) it.next();
            if (method.getName().equals(methodName)) {
                list.add(method);
            }
        }
        return (Method[]) list.toArray(new Method[0]);
    }

    public int getMethodCount(String methodName) {
        return getMethods(methodName).length;
    }

    public Method getMethod(String methodName, Class[] clazz) {
        Method method = null;
        try {
            method = this.clazz.getMethod(methodName, clazz);
        } catch (java.lang.SecurityException e) {
            throw getSecurityException(e);
        } catch (java.lang.NoSuchMethodException e) {
            throw getNoSucheMethodException(e);
        }
        return method;
    }

    private SecurityExceptionWrapper getSecurityException(
            java.lang.SecurityException exception) {
        return new SecurityExceptionWrapper(exception);
    }

    private NoSuchMethodExceptionWrapper getNoSucheMethodException(
            java.lang.NoSuchMethodException exception) {
        return new NoSuchMethodExceptionWrapper(exception);
    }
}