package com.wikihouse.wildcats0201.jdbcchart.reflectionutil;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.MockControl;
import org.easymock.classextension.MockClassControl;

import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.ClassNotFoundExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.InstantiationExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.NoSuchMethodExceptionWrapper;
import com.wikihouse.wildcats0201.jdbcchart.reflectionutil.exception.SecurityExceptionWrapper;

/**
 * <B>ClassWrapperTest </B>.
 * 
 * @author $Author: wildcats $
 * @version $Revision: 2 $
 */
public class ClassWrapperTest extends TestCase {

    private ClassWrapper wrapper = null;

    public void setUp() throws Exception {
        super.setUp();
        this.wrapper = new ClassWrapper(NormalTestClass.class);
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.ClassWrapper(String)'
     */
    public void testClassWrapperString() {
        ClassWrapper wrapper = new ClassWrapper("java.lang.String");
        Class clazz = wrapper.get();
        assertEquals(String.class, clazz);
        try {
            new ClassWrapper("java.ang.String");
            fail();
        } catch (ClassNotFoundExceptionWrapper e) {
        }
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.ClassWrapper(Class)'
     */
    public void testClassWrapperClass() {
        assertEquals(NormalTestClass.class, this.wrapper.get());
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.newInstance()'
     */
    public void testNewInstance() {
        Object obj = this.wrapper.newInstance();
        assertNotNull(obj);
        assertTrue(obj instanceof NormalTestClass);
        ClassWrapper bad = new ClassWrapper(List.class);
        try {
            bad.newInstance();
            fail();
        } catch (InstantiationExceptionWrapper e) {
        }

    }

    public static class NormalTestClass {
        public NormalTestClass() {
        }

        public NormalTestClass(String string) {
        }

        public NormalTestClass(String string, int i) {
        }

        protected NormalTestClass(int i) {
        }

        NormalTestClass(int[] ints) {
        }

        private NormalTestClass(double d) {
        }
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getConstructor(Class)'
     */
    public void testGetConstructorClass() {
        ConstructorWrapper good = this.wrapper.getConstructor(String.class);
        assertNotNull(good);
        assertNotNull(good.get());
        Constructor con1 = good.get();
        assertEquals(1, con1.getParameterTypes().length);
        assertEquals(String.class, con1.getParameterTypes()[0]);

        try {
            this.wrapper.getConstructor(ArrayList.class);
            fail();
        } catch (NoSuchMethodExceptionWrapper e) {
        }

        MockControl control = MockClassControl.createControl(
                ClassWrapper.class, new Class[] { Class.class },
                new Object[] { ClassWrapper.class });
        ClassWrapper classMockWrapper = (ClassWrapper) control.getMock();
        Class[] classes = new Class[] { String.class };
        classMockWrapper.getConstructor(classes);
        control.setThrowable(new SecurityExceptionWrapper(null));
        control.replay();

        try {
            classMockWrapper.getConstructor(classes);
            fail();
        } catch (SecurityExceptionWrapper e) {
        }
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getConstructor(Class[])'
     */
    public void testGetConstructorClassArray() {
        ConstructorWrapper good = this.wrapper.getConstructor(new Class[] {
                String.class, int.class });
        assertNotNull(good);
        assertNotNull(good.get());
        Constructor con1 = good.get();
        assertEquals(2, con1.getParameterTypes().length);
        assertEquals(String.class, con1.getParameterTypes()[0]);
        assertEquals(int.class, con1.getParameterTypes()[1]);

        try {
            this.wrapper.getConstructor(new Class[] { List.class,
                    ArrayList.class });
            fail();
        } catch (NoSuchMethodExceptionWrapper e) {
        }
    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.toString()'
     */
    public void testToString() {

    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getMethods()'
     */
    public void testGetMethods() {

    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getMethods(String)'
     */
    public void testGetMethodsString() {

    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getMethodCount(String)'
     */
    public void testGetMethodCount() {

    }

    /*
     * Test method for
     * 'com.wikihouse.wildcats0201.jdbcchart.reflectionutil.ClassWrapper.getMethod(String,
     * Class[])'
     */
    public void testGetMethod() {

    }

}
